DEFINE CLASS FOXSCRIPT_CONTROL AS Custom
* -------------------------------------------------------------------------------- *
*	FOXSCRIPT_CONTROL
*	=================
*	Desc (short):	FoxScript_Control implements a container to build and execute
*					code created by the user.
*					
*	Autor.......:	Juergen Ulbts (J.U.)
*	Copyright...:	2005 - Juergen Ulbts
*	Version.....:	0.03 - 2005-09-14
*	VFP Version.:	VFP 9.0
*	Description.:	This class allows to compile code within a predefined method
*					body. The code can be entered somewhere in an application and
*					will be compiled. If an error has been found a messagebox
*					will describe the type and location the error occured.
*					The function used to compile the code will return true or false.
*					If the code has been compiled successfully the code should be 
*					saved to a memo field.
*					This class allows to execute the code after successfull 
*					compilation. The create *.FXP will be random and removed after
*					execution!
*
*	History.....:	Format: DATE/DATUM - DEVELOPER/ENTWICKLER (INITIALIEN) - CHANGES/AENDERUNG (multiline/mehrzeilig)
*						2005-09-07 - J.U. -	work started
*						2005-09-10 - J.U. - second version working as expected 
*						                  - added error handling
*						                  - updated method list and descriptions
*						2005-09-14 - J.U. - fixed bug in addCatchedExceptionMsg
*													
*	Methods.....:	init() ---
*					setup() boolean
*
*					assembleUserScriptCode(tcUserCode) boolean
*					compileUserScriptCode(tcPreCheckedCode) boolean
*					executeUserScriptCode() boolean
*
*					cleanUpProgramFiles() boolean
*					deletePRGFile() boolean
*					deleteERRFile() boolean
*					deleteFXPFile() boolean
*
*					addCatchedExceptionMsg(toException) ---
*					addErrorMessage(tcErrorMessageText) ---
*					deleteErrorMessages() ---
*					getErrorMessages() character
*
* -------------------------------------------------------------------------------- *

****************
* SET BY INIT  *
****************
PROTECTED cCRLF						&& CRLF


****************
* SET BY SETUP *
****************
PROTECTED lSetupOk				&& Setup been processed successfully?
PROTECTED cCodeFileStem			&& Filename stem including the full path of the temp prg/err/fxp file
PROTECTED lDebugMode			&& DebugMode
PROTECTED cErrorMessage			&& Error Message


*!* Init for this class 
*!*
*!* Required Parameter(s):
*!*	NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
PROTECTED PROCEDURE init
   DODEFAULT()
   this.cCRLF           = CHR(13)+CHR(10) 
   this.lSetupOk        = .F.
   this.cCodeFileStem   = ""	&& Filename stem including the full path of the temp prg/err/fxp file
   this.cErrorMessage   = ""    && Error Message property
RETURN
ENDPROC



*!* Setup for this class 
*!*
*!* Required Parameter(s):
*!* NONE
*!*	
*!* Optional Parameter(s):     
*!*	NONE
*!*
PROCEDURE setup
LOCAL llok as Boolean
	llok = .T.
	this.cCodeFileStem = ADDBS( SYS(2023) )+ALLTRIM( SUBSTR(SYS(2015), 4, 10) )
	this.lSetupOk = llok && set a marker to show if the setup returned ok or not
RETURN(llok)
ENDPROC



*!* Assemble User Script Code
*!* This function assembles the code from the user and the predefined
*!* code. It does some checks for prohibited variables and functions.
*!* If so far everything is ok it calls the function to compile the code!
*!* That code returns if the compile has been successfull.
*!* Error messages can be retrieved by calling getErrorMessages() which
*!* includes compile errors.
*!* The *.prg and the *.err files are automatically removed, and leave
*!* *.fxp untouched if it has been created until it will be executed or
*!* removed manually by calling the CleanUpProgramFiles().
*!*
*!* Required Parameter(s):
*!* tcUserCode = The code the user has written
*!*	
*!* Optional Parameter(s):     
*!*	NONE
*!*
FUNCTION assembleUserScriptCode
LPARAMETERS tcUserCode
LOCAL llok as Boolean, lcPreCheckedCode
	llok = .T.
	lcPreCheckedCode = ""
	this.deleteErrorMessages() && clear error messages	
	
	IF this.lSetupOk THEN
		IF VARTYPE(tcUserCode) == "C" AND NOT EMPTY(tcUserCode) THEN
			&& some basic checks...
			&& no 'RETURN' allowed to jump out of the code early
			&& ...the lcOldSel may not get executed which could 
			&& be really bad if it's not handled where the code is
			&& going to be executed...
			&& BETTER SAVE THAN SORRY!
			IF (OCCURS('RETURN', UPPER(tcUserCode)) > 0) THEN
				this.addErrorMessage("assembleUserScriptCode(): "+this.cCRLF+;
						"The code contains the string/substring 'RETURN' but;
						 this text is not allowed!")
				llok = .F.
			ENDIF
			
			&& no 'lcOldSel' allowed to save the content of this variable 
			&& set by the surrounding code block
			IF (OCCURS('LCOLDSEL', UPPER(tcUserCode)) > 0) THEN
				this.addErrorMessage("assembleUserScriptCode(): "+this.cCRLF+;
						"The code contains the string/substring 'lcOldSel' but;
						 this text is not allowed!")
				llok = .F.
			ENDIF
			
			IF llok THEN
				lcPreCheckedCode =	lcPreCheckedCode+;
									'PROCEDURE userScriptCode'+this.cCRLF+;
									'LOCAL lcOldSel'+this.cCRLF+;
									'	lcOldSel = STR(SELECT(0))'+this.cCRLF+;
									tcUserCode+this.cCRLF+;
									'	SELECT &lcOldSel'+this.cCRLF+;
									'RETURN'

				llok = this.compileUserScriptCode(lcPreCheckedCode)
			ENDIF
			
		ELSE
			llok = .F.
		ENDIF
		
	ELSE && setup check failed
		llok = this.lSetupOk
		this.addErrorMessage("Es gabe einen Fehler im Setup der FoxScript Kontroll Komponente. Die weitere Verarbeitung wurde gestoppt!", 16, "Fehler") && Error sign
	ENDIF
RETURN llok
ENDPROC



*!* CompileUserScriptCode
*!* This function compiles the code and returns 
*!*
*!* Required Parameter(s):
*!* tcPreCheckedCode = The prechecked sourcecode
*!*	
*!* Optional Parameter(s):     
*!*	NONE
*!*
PROTECTED FUNCTION compileUserScriptCode
LPARAMETERS tcPreCheckedCode
LOCAL llok as Boolean, lcTmpCodeFile
	llok = .T.
	lcTmpCodeFile = ""

	IF VARTYPE(tcPreCheckedCode) == "C" AND NOT EMPTY(tcPreCheckedCode) THEN
		&& build sourcecode file and save the 
		lcTmpCodeFile = this.cCodeFileStem+".PRG"
		STRTOFILE(tcPreCheckedCode, lcTmpCodeFile)
		
		&& compile code
		COMPILE &lcTmpCodeFile
					
		&& if error...add error message to error messages for later processing
		lcTmpCodeFile = this.cCodeFileStem+".ERR"	
		llok = NOT FILE(lcTmpCodeFile)
		
		IF !llok THEN && *.ERR file found
			this.addErrorMessage( "Der Code konnte nicht fehlerfrei bersetzt werden: "+this.cCRLF+;
								FILETOSTR(lcTmpCodeFile) )
		ENDIF				
		
		&& Delete *.PRG file
		this.DeletePRGFile()
		
		&& Delete *.ERR file
		this.DeleteERRFile()
		
		lcTmpCodeFile = this.cCodeFileStem+".FXP"
		llok = llok AND FILE(lcTmpCodeFile) && program compiled successfully?
	ELSE
		llok = .F.
	ENDIF
	
RETURN llok
ENDFUNC



*!* ExecuteUserScriptCode
*!* This function executes the compiled the code
*!*
*!* Required Parameter(s):
*!* NONE
*!*	
*!* Optional Parameter(s):     
*!*	NONE
*!*
FUNCTION executeUserScriptCode
LOCAL llok as Boolean, lcTmpCodeFile as String
	llok = .T.
	lcTmpCodeFile = this.cCodeFileStem+".FXP"
	
	IF FILE(lcTmpCodeFile) THEN
		TRY
			&& excute the code
			DO userScriptCode IN &lcTmpCodeFile
		CATCH TO loError
			llok = .F.
			IF VARTYPE(loError) == "O" AND NOT ISNULL(loError) THEN
				loError.UserValue = lcTmpCodeFile
				this.addCatchedExceptionMsg(loError)
			ENDIF
		ENDTRY
	ELSE
		llok = .F.
	ENDIF
	
RETURN llok
ENDFUNC



*!* cleanUpProgramFiles
*!* This method tries to delete all files that could be created when a program file
*!* will be compiled (*.prg, *.err, *.fxp).
*!*
*!* Required Parameter(s): 
*!* NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
FUNCTION cleanUpProgramFiles
LOCAL llok as Boolean
	&& Delete *.PRG file
	llok = this.DeletePRGFile()
	
	&& Delete *.ERR file
	llok = llok AND this.DeleteERRFile()
	
	&& Delete *.FXP file
	llok = llok AND this.DeleteFXPFile()
RETURN llok
ENDFUNC



*!* deletePRGFile
*!* This method deletes the *.prg file if found and returns true if it could be deleted.
*!*
*!* Required Parameter(s): 
*!* NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
FUNCTION deletePRGFile
LOCAL llok as Boolean, lcTmpCodeFile
LOCAL loError as Exception 
	llok = .T.
	lcTmpCodeFile = this.cCodeFileStem+".PRG"
	IF FILE(lcTmpCodeFile) THEN
		TRY
			&& ERASE/DELETE the file
			ERASE &lcTmpCodeFile
		CATCH TO loError
			llok = .F.
			IF VARTYPE(loError) == "O" AND NOT ISNULL(loError) THEN
				loError.UserValue = lcTmpCodeFile
				this.addCatchedExceptionMsg(loError)
			ENDIF
		ENDTRY
	ENDIF
RETURN llok
ENDFUNC



*!* deleteERRFile
*!* This method deletes the *.err file if found and returns true if it could be deleted.
*!*
*!* Required Parameter(s): 
*!* NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
FUNCTION deleteERRFile
LOCAL llok as Boolean, lcTmpCodeFile
LOCAL loError as Exception 
	llok = .T.
	lcTmpCodeFile = this.cCodeFileStem+".ERR"
	IF FILE(lcTmpCodeFile) THEN
		TRY
			&& ERASE/DELETE the file
			ERASE &lcTmpCodeFile
		CATCH TO loError
			llok = .F.
			IF VARTYPE(loError) == "O" AND NOT ISNULL(loError) THEN
				loError.UserValue = lcTmpCodeFile
				this.addCatchedExceptionMsg(loError)
			ENDIF
		ENDTRY
	ENDIF
RETURN llok
ENDFUNC


*!* deleteFXPFile
*!* This method deletes the *.fxp file if found and returns true if it could be deleted.
*!*
*!* Required Parameter(s): 
*!* NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
FUNCTION deleteFXPFile
LOCAL llok as Boolean, lcTmpCodeFile
LOCAL loError as Exception 
	llok = .T.
	lcTmpCodeFile = this.cCodeFileStem+".FXP"
	IF FILE(lcTmpCodeFile) THEN
		TRY
			&& ERASE/DELETE the file
			ERASE &lcTmpCodeFile
		CATCH TO loError
			llok = .F.
			IF VARTYPE(loError) == "O" AND NOT ISNULL(loError) THEN
				loError.UserValue = lcTmpCodeFile
				this.addCatchedExceptionMsg(loError)
			ENDIF
		ENDTRY
	ENDIF
RETURN llok
ENDFUNC


*!* addCatchedExceptionMsg 
*!* This method adds an error message with the content of the exception 
*!* object to this objects cErrorMessage property.
*!*
*!* Required Parameter(s): 
*!* toException = Exception object
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
PROTECTED PROCEDURE addCatchedExceptionMsg
LPARAMETERS toException
	IF VARTYPE(toException) == "O" AND NOT ISNULL(toException) THEN
		this.addErrorMessage("Catched Error in: "+toException.Procedure+this.cCRLF+;
			"ErrorNo....: "+TRANSFORM(toException.ErrorNo)+this.cCRLF+;
			"Line.......: "+TRANSFORM(toException.LineNo)+this.cCRLF+;
			"LineContent: "+toException.LineContents+this.cCRLF+;
			"UserValue..: "+toException.UserValue+this.cCRLF)
	ENDIF
ENDPROC


*!* addErrorMessage 
*!* This method adds an error message to the cErrorMessage property
*!*
*!* Required Parameter(s): 
*!* tcErrorMessageText = Error message that is added to this.cErrorMessage
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
PROTECTED PROCEDURE addErrorMessage
LPARAMETERS tcErrorMessageText
   IF VARTYPE(tcErrorMessageText) == "C" THEN
      IF EMPTY(tcErrorMessageText) THEN && delete old messages
         this.cErrorMessage = ""
      ELSE
         IF (LEN(this.cErrorMessage) == 0) THEN
            this.cErrorMessage = tcErrorMessageText
         ELSE
            this.cErrorMessage = this.cErrorMessage + this.cCRLF + tcErrorMessageText
         ENDIF
      ENDIF
   ELSE && error in error message
      this.cErrorMessage = this.cErrorMessage + this.cCRLF + "Error message wasn't from type Character!"
   ENDIF
RETURN
ENDPROC


*!* deleteErrorMessages
*!* This method deletes all previously saved error messages from the cErrorMessage property.
*!*
*!* Required Parameter(s): 
*!* NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
PROCEDURE deleteErrorMessages
	this.cErrorMessage = ""
RETURN
ENDPROC


*!* getErrorMessages
*!* This method returns the error text stored in the cErrorMessage property.
*!*
*!* Required Parameter(s): 
*!* NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
FUNCTION getErrorMessages
	RETURN(this.cErrorMessage)
ENDPROC

ENDDEFINE && END OF CLASS DEFINITION
